/*
 * Argument support
 * ----------------
 * (c) 2001 by David Gerber <zapek@meanmachine.ch>
 * All Rights Reserved
 *  
 * $Id: args.c,v 1.1 2013/01/14 21:47:38 jacadcaps Exp $
 */

#include "globals.h"

/* public */
#include <dos/rdargs.h>
#include <exec/memory.h>
#include <workbench/workbench.h>
#include <workbench/startup.h>
#include <proto/dos.h>
#include <proto/exec.h>
#include <proto/icon.h>
#include <strings.h>

/* private */
#include "args.h"

static struct RDArgs *rda, *rdax;
static struct DiskObject *diskobj;

static char *help = "Hehe, you found the hidden message.";
static char *templ = "HOST/A,HOST2/K,HOST3/K,HOST4/K,PASSWORD/K,PASSWORD2/K,PASSWORD3/K,PASSWORD4/K,PORT/K/N,PORT2/K/N,PORT3/K/N,PORT4/K/N,HOTKEY_CONNECT/K,HOTKEY_CONNECT2/K,HOTKEY_CONNECT3/K,HOTKEY_CONNECT4/K,CX_PRIORITY/K/N,NODEADKEY/S";

struct arguments args;

/*
 * Support functions
 */

/*
 * Returns TRUE if the tooltype is enabled
 * and FALSE if it's disabled (BLAH). Also
 * check our own argument list to see if
 * the tooltype is an external one which is
 * handled by the Workbench, MCP or so..
 */
static ULONG is_enabled(STRPTR s)
{
	while (*s)
	{
		if (*s == ' ') /* skip front spaces */
		{
			s++;
		}
		else if (*s == '(')
		{
			return (FALSE);
		}
		else
		{
			/* check if it's one of our argument */
			STRPTR p, q;
			int match = FALSE;
			STRPTR r = 0;
			int is_equal = TRUE;

			p = q = templ;

			/* only compare with the string before '=' (and without spaces) */
			if (r = strchr(s, '='))
			{
				while (r[-1] == ' ')
				{
					is_equal = FALSE;
					r--;
				}
				*r = '\0';
			}

			while (*p)
			{
				if (*p == '/')
				{
					*p = '\0';

					DB(("comparing %s against %s..\n", q, s ));
					if (!stricmp(q, s))
					{
						DB(("matches!\n"));
						match = TRUE;
					}
					
					*p = '/';
					
					if (match)
					{
						break;
					}
					
					if (p = strchr(p, ','))
					{
						q = p + 1;
					}
					else
					{
						break;
					}
				}
				p++;
			}

			if (r)
			{
				*r = is_equal ? '=' : ' ';
			}

			if (match)
			{
				return (TRUE);
			}
			else
			{
				return (FALSE);
			}
		} 
	}
	return (FALSE);
}


static ULONG argcat(STRPTR d, STRPTR s)
{
	if (is_enabled(s))
	{
		/*
		 * Position d properly.
		 */
		while (*d)
		{
			d++;
		}

		/*
		 * Copy until the end or
		 * '=' is reached. Remove spaces.
		 */
		while (*s)
		{
			if (*s == ' ')
			{
				DB(("skipping space\n"));
				s++;
			}
			else if (*s == '=')
			{
				int has_space = FALSE;

				*d++ = *s++;

				DB(("'=' reached\n"));

				while (*s) /* skip leading spaces */
				{
					if (*s != ' ')
						break;
					s++;
				}
				
				DB(("leading spaces skipped\n"));

				/*
				 * Find out if there's a space in
				 * the string so that we put '"'s around.
				 */
				if (strchr(s, ' '))
				{
					DB(("extra \"s needed\n"));
					*d++ = '"';
					has_space = TRUE;
				}

				while (*s)
				{
					DB(("plain string copy\n"));
					*d++ = *s++;
				}

				if (has_space)
				{
					*d++ = '"';
				}
				DB(("ok, finished\n"));
				break;
			}
			else
			{
				*d++ = *s++;
			}
		}
		*d = '\0';
		return (TRUE);
	}
	else
	{
		return (FALSE);
	}
}


/*
 * Starts the argument parsing. Supply NULL as 's'
 * to have it use the normal CLI input.
 */
ULONG start_args(int argc, char **argv)
{
	if (rdax = (struct RDArgs *)AllocDosObject(DOS_RDARGS, NULL))
	{
		int ret = FALSE;
		STRPTR t = 0;

		if (!argc)
		{
			struct WBStartup *WBMsg = (struct WBStartup *)argv;

			DB(("workbench startup..\n"));

			/*
			 * Count the size of the tooltype array
			 */
			if (WBMsg->sm_ArgList && WBMsg->sm_ArgList->wa_Name)
			{
				ULONG sz = 1; /* '\n' */
				BPTR l = CurrentDir(WBMsg->sm_ArgList->wa_Lock);
				
				DB(("got CurrentDir()\n"));

				if (diskobj = GetDiskObject(WBMsg->sm_ArgList->wa_Name))
				{
					UBYTE **argarray = (UBYTE **)diskobj->do_ToolTypes;
				
					DB(("got diskobject\n"));

					while (*argarray)
					{
						DB(("sz == %ld\n", sz));
						if (is_enabled(*argarray))
						{
							sz += strlen(*argarray) + 3; /* NULL term + possible "" */
						}
						argarray++;
					}
					DB(("final sz == %ld\n", sz));

					if (t = AllocVec(sz + 1, MEMF_ANY))
					{
						t[0] = '\0';
						argarray = (UBYTE **)diskobj->do_ToolTypes;

						while (*argarray)
						{
							if (argcat(t, *argarray++))
							{
								if (*argarray)
								{
									DB(("handling..\n"));
									strcat(t, " ");
								}
							}
						}
						strcat(t, "\n"); /* ReadArgs() needs that */
						DB(("final strings: |%s|\n", t));
					}
				}
				CurrentDir(l);

				if (!t)
				{
					DB(("error\n"));
					return (0);
				}
				rdax->RDA_Source.CS_Buffer = t;
				rdax->RDA_Source.CS_Length = sz;
				rdax->RDA_Source.CS_CurChr = 0; 
				DB(("ok, ReadArgs() initialized, going to use it on string (len: %ld): %s\n", rdax->RDA_Source.CS_Length, rdax->RDA_Source.CS_Buffer));
			}
		}
		rdax->RDA_ExtHelp = help;
		if (rda = ReadArgs(templ, (LONG *)&args, rdax))
		{
			DB(("all fine\n"));
			ret = TRUE;
		}

		if (t)
		{
			FreeVec(t);
		}
		DB(("returning..\n"));
		return (ret);
	}
	return (0);
}


void end_args(void)
{
	if (diskobj)
	{
		FreeDiskObject(diskobj);
	}

	if (rda)
	{
		FreeArgs(rda);
	}

	if(rdax)
	{
		FreeDosObject(DOS_RDARGS, rdax);
	}
}
